/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.ItemLevelAction");

(function() {
var _dojo = dojo;
var isIE7 = _dojo.isIE == 7;
var InlineStyle = (isIE7) ? 'inline' : 'inline-block';
var TableCellItemLevelAction = com.ibm.asq.common.web.ui.internal.widgets.TableCellItemLevelAction;

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ImageCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{    
    // Summary:
    //    Renders the image cell content. The following cellData can be expected. 
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex)
    {
        var imgSrc = extendedDisplayInfo ? (extendedDisplayInfo['mouseOutImage'] || extendedDisplayInfo['image'] || extendedDisplayInfo['decorateImage']) : null;
        var img = imgSrc ? _dojo.create("img") : _dojo.create('span');
        var tooltip = cellData;
        var clickable = false;
        
        if(imgSrc) {
            // only clickable if it's an image. decorateImage cannot.
            clickable = (extendedDisplayInfo['image'] != undefined);
            //when adding Tooltip there is no need to use the alt attribute of the images in IE8 is duplicating the information
			//Set the title, and therefore the tooltip that is displayed for the image to be:
			//1.  Extended tooltip info if any
			//2.  Extended alt info if any
			//3.  The cellData.
            var tooltipOrTooltipInfo = extendedDisplayInfo['tooltip'] || extendedDisplayInfo['tooltipInfo'];
            img.title = tooltipOrTooltipInfo ? tooltipOrTooltipInfo : extendedDisplayInfo['alt'] ? extendedDisplayInfo['alt'] : tooltip ? tooltip : "Image";//$NON-NLS
	            
			img.alt = extendedDisplayInfo['alt'] ? extendedDisplayInfo['alt'] : tooltipOrTooltipInfo ? tooltipOrTooltipInfo : "";
			if(img.alt == "")
				img.alt = "Image";//$NON-NLS
            img.src = imgSrc;
            
            if(extendedDisplayInfo['tooltip'])
                tooltip = extendedDisplayInfo['tooltip'];
			if (extendedDisplayInfo['mouseOverImage']){
			
    	    	var x = extendedDisplayInfo['mouseOutImage'] || extendedDisplayInfo['image'];
    	    	var srcOut = _dojo.isString(x) ? x : x.toString();
    	    	 
    	    	x = extendedDisplayInfo['mouseOverImage'];
    	    	var srcOver = _dojo.isString(x) ? x : x.toString();
    	    	
    	    	this.connect(img, "onmouseout", dojo.hitch({'srcOut': srcOut},this.outOfImage));
	        	this.connect(img, "onmouseover", dojo.hitch({'srcOver': srcOver},this.overImage));
	        	this.connect(img, "onfocus", dojo.hitch({'srcOver': srcOver},this.overImage));
	        	this.connect(img, "onblur", dojo.hitch({'srcOut': srcOut},this.outOfImage));
    	    }
        }
        
        if (clickable && extendedDisplayInfo &&  extendedDisplayInfo['href']) {
            // display image as link only if the table is not displayed in a popup
            if (!viewer.displayInPopup || !viewer.isInPicker) {
				if(imgSrc && isIE7){
					//Specifically for IE 7 ALT text are showing on hover when external tooltip is added.
					img.alt = "";
					img.title = ""; //avoid AVT issue here.
				}  
                
                var anchor = _dojo.create("a");
                anchor.href = extendedDisplayInfo.href;
                anchor.appendChild(img);
                return anchor;
            }
        } else {
            if(imgSrc && isIE7){
				//Specifically for IE 7 ALT text are showing on hover when external tooltip is added.
				img.alt = "";
				img.title = ""; //avoid AVT issue here.
			}
        }

        return img;
    },
    
    overImage: function(e)
    {	
    	if (this.srcOver)
    		e.target.src = this.srcOver;
    },
    
    outOfImage: function(e)
    {
    	if (this.srcOut)
    		e.target.src = this.srcOut;
    },
    
    // convenient method to draw an image for other cell renderer types.
    addImageRendering: function(node, viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint, /*optional*/ itemLevelAction)
    {
        var imgNode = null;
        var isCompact = viewer.isCompactMode;
        
        if (extendedDisplayInfo) {
            if (extendedDisplayInfo['decorateImage']) {
                imgNode = this.renderCell(viewer, null, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint);
                _dojo.style(imgNode, {
                    'verticalAlign': 'middle',
                    'margin': '3px 0px'
                });
            }
            else if (extendedDisplayInfo['actionImage']) {
                var action = new TableCellItemLevelAction({
                    imgSrc: extendedDisplayInfo['actionImage'],
                    alt: extendedDisplayInfo['alt'],
                    label: extendedDisplayInfo['label'],
                    actionData: rowObject,
                    imageLockVisible: extendedDisplayInfo['alwaysShowActionImage'] ? true : false,
                    actions: itemLevelAction ? extendedDisplayInfo['itemActions'] : extendedDisplayInfo['actions'],
                    actionHandler: extendedDisplayInfo['actionHandler']
                });
                imgNode = action.domNode;
                this.registerWidget(action);
                if (!extendedDisplayInfo['alwaysShowActionImage']) {
                // Connect node mouse hover events to show/hide inline action image
                this.connect(node, "onmouseover", function() {
            		action.set("lockImage", true);
            	});
            	this.connect(node, "onmouseout", function() {
            		action.set("lockImage", false);
            	});
            }
        }
        }
        if(imgNode) {
            var holder = _dojo.create('div', {
                style: {
                    'display': InlineStyle,
                    'whiteSpace': isCompact ? 'nowrap' : ""
                }
            });
            if (extendedDisplayInfo['actionImage']){            	
            	_dojo.style(holder, "position", "relative");
            	_dojo.style(holder, "display", "");
                _dojo.place(node, holder);
                _dojo.place(imgNode, holder);            	
            }else{
            	 _dojo.place(imgNode, holder);
                 _dojo.place(node, holder);
            }

            _dojo.style(node, {
                'marginLeft': '0px',
                'left': '5px',
                'verticalAlign': 'middle',
                'position': 'relative',
                'display': isCompact ? InlineStyle : ""
            });
            node = holder;            
        }
        return node;
    }
});

})();
