/****************************************************************************** 
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2014. All Rights Reserved.
 *  
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.RichHoverCellRenderer");

dojo.require("com.ibm.asq.common.web.ui.internal.util.MessageUtil");
dojo.require("com.ibm.asq.common.web.ui.internal.util.TooltipBuilder");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer");
dojo.require("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants");
dojo.require("com.ibm.asq.common.web.ui.internal.view.common.ImageCache");

dojo.require("jazz.ui.ResourceLink");

(function() {
var _d = dojo;
var IsIE = _d.isIE;
var IsChrome = _d.isChrome;
var _textContent = "innerHTML"; 
var TableConstants = com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableConstants;
var ImageCache     = com.ibm.asq.common.web.ui.internal.view.common.ImageCache;
var TooltipBuilder = com.ibm.asq.common.web.ui.internal.util.TooltipBuilder;

// use max width to avoid resource link's text label to extend to the cell width.
var WidthStyle = "maxWidth";
var IconWidth = 24;

// Override the base resource link implementation to make its' label display
// to conform to the width constraint of a table cell.
//
dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ResourceLink", jazz.ui.ResourceLink,
{
    // @override jazz.ui.ResourceLink - to properly clip the resource link's text
    // when the table is in compact mode.
    _applyTitle: function(title) {
        if(!this.domNode){
            this.buildRendering();
        }       
        if (this.arrowNode && this.arrowNode.parentNode)
            this.arrowNode.parentNode.removeChild(this.arrowNode);
        
        // wrap the text in a div to properly clip the text content.
        // CSS text clipping requires a div
        if(this.isCompactCell) {
            // clear the old label first, could have a 'loading' shown.
            this.domNode.innerHTML = "";
            var innerLabel = _d.create(IsIE ? 'a' : 'div', {'className': "rich-hover-clip-cell"});
            
            title = this.restoreTags(title);
            innerLabel[_textContent] = title;
            
            if (IsIE) {
                // Workaround the IE bug that it does not offer the open link in new tab/window options on
                // right click if there is display:block element.  The innerLabel must be display:block in
                // order to have trim text to work.  Create <a> instead of <div> for IE using the same href.
                innerLabel.href = this.domNode.href;
            }

            // the text decoration for links isn't inherited by the DIV in the link (domNode)
            // have to actively add the decoration when hover.
            if(IsIE || IsChrome) {
                this.connect(innerLabel, 'onmouseover', function(e) {
                    if(e.target) {
                        dojo.style(e.target, 'textDecoration', "underline");
                    }
                });
                this.connect(innerLabel, 'onmouseout', function(e) {
                    if(e.target) {
                        dojo.style(e.target, 'textDecoration', "none");
                    }
                });
            }
            _d.place(innerLabel, this.domNode, 'first');
            // render title will add the css to clip text.
            this.renderTitle(innerLabel, title);  
        }
        else {
            // no-clipping. Just apply the title to the link itself.
            this.domNode[_textContent] = title;
        }
        if (this.arrowNode)
            this.domNode.appendChild(this.arrowNode);
    }
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.RichHoverCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.TableCellRenderer,
{   
    // Summary:
    //    Renders the cell content which need a rich hover to display the content. 
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var node = null;
        var doc = document;
        // render an image in front of the link if extended data specified an decorative image.
        var hasImage = this._hasDecorateImage(extendedDisplayInfo);
        var imageOnly = false;
            
        // if we have a decorate image, don't create the image for the resource link
        var createImage = !(extendedDisplayInfo && (extendedDisplayInfo['decorateImage'] || extendedDisplayInfo['image'] || extendedDisplayInfo['noImage'])); 
        var adjustedCellWidthHint = cellWidthHint - ((hasImage) ? IconWidth : 0);
        
        //if we explicitly decided to display a simple hover (no presentationUri, no uri)
        if(extendedDisplayInfo.simpleHover && extendedDisplayInfo.hoverContent){
        	node = document.createElement("A");
 		    node.href = "javascript:{}";
 		    node.innerHTML = extendedDisplayInfo.label;	        	    
 			TooltipBuilder.addTooltip(extendedDisplayInfo.hoverContent, node);
 			return node;
        }
        
        // no extended info or uri, degenerate to a label/image.
        if (!extendedDisplayInfo || !extendedDisplayInfo.uri) {
            this.showTooltipTitleInCompactMode = true;
            if (extendedDisplayInfo.image) {
                node = doc.createElement("img");
                node.alt = extendedDisplayInfo.alt ? extendedDisplayInfo.alt : "Image";//$NON-NLS
                node.src = extendedDisplayInfo.image;
                this._attachTooltipToCell(node, extendedDisplayInfo.tooltip);   
                imageOnly = true;
            }
            else {
                node = doc.createElement("div");
                this._renderCellText(viewer, cellData, extendedDisplayInfo, node, adjustedCellWidthHint);
            }
        }
        else {
            // don't show the default tool tip, we're showing the rich hover already
            this.showTooltipTitleInCompactMode = false;
            
            node = this._createLinkNode(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, adjustedCellWidthHint, createImage, true);
            if (hasImage) {
                var imageRenderer = viewer.getCellRenderer(TableConstants.RENDER_TYPE_IMAGE);
                node = imageRenderer.addImageRendering(node, viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint);
            }
        }
        // allow the wrapper node to resize
        if (!imageOnly) {
            _d.addClass(node, TableConstants.CELL_RESIZE_MARKER_CLASS_NAME);
        }
        return node;
    },
    
    _createLinkNode: function(viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint, createImage, resizable, useShortTitle)
    {
        if (rowObject && rowObject.snapshotInfo && extendedDisplayInfo && extendedDisplayInfo.presentationUri && extendedDisplayInfo.presentationUri[0] === '#'){
            // the table viewer is displayed part of a test artifact that is a snapshot
            // we need to not render a hover link but just a normal link for rqm test artifact references
            var linkRenderer = viewer.getCellRenderer(TableConstants.RENDER_TYPE_LINK);
            var linkInfo = {
                    href: extendedDisplayInfo.presentationUri,
                    target: extendedDisplayInfo.target,
                    linkType: "undefined",
                    tooltip: extendedDisplayInfo.alt
            };
            return linkRenderer.renderCell(viewer, cellData, linkInfo, cellModifier, rowObject, columnIndex, cellWidthHint);
        }

        var doc = document;
        var non_link = viewer.displayInPopup || viewer.isInPicker; // disable the link when the host table is in a dialog
        var node = doc.createElement("A");

// TODO: open content in same window until user pref is implemented to support open in external window/tab
//        if (!extendedDisplayInfo.openInTab) {
//            //Set openInTab as true in extendedDisplayInfo if want to open the uri within the QM page
//            node.target = "_blank";
//        }
        var islazyFetch = extendedDisplayInfo.lazyFetch || false;
        var label = extendedDisplayInfo.label;

        // Interpret noUpdateLabel specified as no live update
        // of the label.
        var noUpdate = (extendedDisplayInfo.noUpdateLabel == undefined && label) ? true :  extendedDisplayInfo.noUpdateLabel;
        
        // if this is live update of label, or it's not specified,
        // make it live update. This ensure that in compact mode
        // the label would be drawn, either immediately, or by 
        // live update.
        if(!noUpdate){
            islazyFetch = false;
        }
        if(extendedDisplayInfo.target)
            node.target = extendedDisplayInfo.target;
        
        var compact = (viewer.isCompactMode && typeof cellWidthHint != "undefined" && dojo.isString(label));
        
        // showing an image. No need to handle clipping of label.
        if (extendedDisplayInfo.image) {
            var img = doc.createElement("img");
            img.alt = extendedDisplayInfo.alt ? extendedDisplayInfo.alt : "Image";//$NON-NLS  
            img.src = extendedDisplayInfo.image;
            node.appendChild(img);
            label = "";
            noUpdate = true;
            islazyFetch = true;
            compact = false;
        }
        //If label is a number, it wouldn't into the extendedDisplayInfo.imgSrchandleCompactCellText Method (due to the isString validation),
        // hence the numbers will be in "raw" mode, making IE to act funny and wrap longer numbers into a single cell (23741).
        else if (!isNaN(label)) {
            node.style.whiteSpace = "nowrap";
            label += "";
            compact = false;
        }
                
        // <td> in IE does not honor max-width style and the column will expand to the width of the unclipped
        // label width.  Add a fixed width wrapper <div> to hold the link node and keep the <td> from expanding.
        // Create an css id marker on the div to allow the cell to be dynamically resized.
        var wrapperNode = node;
        var hasWrap = false;
        var resLinkIcon = null;
        if (compact) {
            wrapperNode = _d.create('div');
            if (_d.isIE == 7) {
                dojo.style(wrapperNode, "overflowX", 'hidden');
            }
            var cwidth = Math.max(8, cellWidthHint);
            _d.style(wrapperNode, "width", cwidth + 'px');
            _d.place(node, wrapperNode, 'first');
            hasWrap = true;
        }
        
        // create a default image for the resource link
        if(createImage) {
            var newWrap = hasWrap ? wrapperNode : _d.create('span');
            resLinkIcon = _d.create('img', {
                'className': 'cell-dec-icon'
            });
            var alt = extendedDisplayInfo.alt || "Image";//$NON-NLS  
            resLinkIcon.src = extendedDisplayInfo.fallbackImage ? extendedDisplayInfo.fallbackImage : ImageCache.EMPTY_IMAGE_SM;
            //IE automatically sets width=2 and height=2 for icon ImageCache.EMPTY_IMAGE_SM which is 2px*2px.
            //After the icon for the resource link is loaded, when reset the src of the image element, sometime the width and height of the 
            //image element is not reset, so the icon is only partially visible 2px*2px.
            //Fix the problem by set the width and height with the default size.
            //See: Defect 103593: QM Print View does not display CCM artifacts in IE 10
            if(_d.isIE){
            	_d.style(resLinkIcon, 'width', '16px');
            	_d.style(resLinkIcon, 'height', '16px');
            }
            resLinkIcon.title = alt;
            resLinkIcon.alt = alt;
            _d.place(resLinkIcon, newWrap, 'first');
            if (!hasWrap) {
                _d.place(wrapperNode, newWrap);
                wrapperNode = newWrap;
            }
        }
        
        label = this.replaceHTMLChars(label);    
        var resourceLinkArgs = {
            uri: extendedDisplayInfo.uri,
            presentationUri: non_link ? "javascript:{}" : extendedDisplayInfo.presentationUri,
            
            // don't provide the label yet if we're in compact mode, we
            // will call the applyTitle to process the label.
            label: (compact) ? "" : label,
            lazyFetch: islazyFetch,
            retainLinkText: noUpdate,
            isCompactCell: compact,
            useAbbreviation: useShortTitle,
			// pass the icon node to ResourceLink for it to update.
            iconNode: resLinkIcon,
    /*
     * see jazz.ui.resoucelink
     *      lazyFetch: (boolean) If set to true, the Resource Link will 
     *          will not fetch the compact rendering document until the user                
     *          hovers over the link.  Also, if this option is set to true, 
     *          the "retainLinkText" parameter will automatically be set to true, 
     *          and the "iconNode" parameter will be ignored.  By default, 
     *          this parameter is set to false, and the compact rendering document 
     *          will be aggressively fetched when the page loads.  
     *                  
     */        
          
            
            
            // have the link call us back to render the updated label. We need to clip the
            // text to stay within the cell's width.
            renderTitle: dojo.hitch(this, function(viewer, cellWidthHint, resizable, node, title){
                this._handleCompactCellText(viewer, title, null, node, cellWidthHint - (createImage ? IconWidth : 0), WidthStyle, resizable);
            }, viewer, cellWidthHint, resizable),
            restoreTags: function(value){
                var rawText = value;
                rawText = rawText.replace(/\&amp;/gi, "&");
                rawText = rawText.replace(/\&tilde;/gi, "~");
                rawText = rawText.replace(/\&quot;/gi, "\"");
                rawText = rawText.replace(/\&circ;/gi, "^");
                rawText = rawText.replace(/\&nbsp;/gi, " ");
                rawText = rawText.replace(/<br\/>/gi, "\n");
                rawText = rawText.replace(/\&#09;/g, "\t");
                return rawText;
            }
        };

        var resLink = new com.ibm.asq.common.web.ui.internal.widgets.tableViewer.ResourceLink(resourceLinkArgs, node);
        
        // In compact mode and a label, process the
        // label on the link now.
        if(compact && label) {
            resLink._applyTitle(label);
        }
        this.registerWidget(resLink);
        return wrapperNode;
    },
    
    replaceHTMLChars: function(string) {
        return string.replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/\n/g, "<br>").replace(/&;/g,"&amp;"); 
    }    
});

dojo.declare("com.ibm.asq.common.web.ui.internal.widgets.tableViewer.RichHoverListCellRenderer", 
             com.ibm.asq.common.web.ui.internal.widgets.tableViewer.RichHoverCellRenderer,
{    
    // Summary:
    //    Renders the cell content of a list of rich hovers. Expected displayInfo:
    //    {
    //          items: [
    //                    {
    //                      label:  
    //                      uri:  
    //                      presentationUri: 
    //                      // true to use a number in place of the the link's label if the list contains more then 1 link
    //                      useNumeric: 
    //                    },
    //                    ...
    //                  ]
    //    }
    //    
    renderCell: function(viewer, cellData, extendedDisplayInfo, /*table.CellModifier*/ cellModifier, rowObject, columnIndex, cellWidthHint)
    {
        var doc = document;
        var node = _d.create('div');
        var items = extendedDisplayInfo.items;
        var nItems = items ? items.length : 0;
        var atLast = nItems - 1;
        var useNumeric = extendedDisplayInfo.useNumericLabel && nItems > 1;
        var hasImage = this._hasDecorateImage(extendedDisplayInfo);
        
        // no title popup on rich hovers
        this.showTooltipTitleInCompactMode = false;
        
        var nodeWidth = useNumeric ? hasImage ? "90%" : "" : "";
       	_d.style(node, 'width', nodeWidth);
        
        // we always want to wrap the list of numbered items. By not specifying a
        // width hint (undefined) for each of the links, this would not force
        // a width on each of the links. Otherwise in IE, with compact turned on, each link
        // would render in separate line. 
       	// Single link width = cell width - left margin - icon width
        var linkWidth = (!useNumeric) ? (cellWidthHint - 5 - (hasImage ? IconWidth : 0)) : undefined;
        
        for (var i = 0; i < nItems; i++) {
            var itemData = items[i];
            if(useNumeric) {
                itemData.label = MessageUtil.getMessage(MessageBundle.COMMON, "labelParenthesized", (i+1));
            }
			itemData.noUpdateLabel = useNumeric || extendedDisplayInfo.noUpdateLabel;
			
			if(itemData.missingBackLink == true){
				var aLink = this._createLinkNode(viewer, cellData, itemData, cellModifier, rowObject, columnIndex, linkWidth, false, /*don't add resize marker*/false, useNumeric);
            
	            var imageRenderer = viewer.getCellRenderer(TableConstants.RENDER_TYPE_IMAGE);
	            
	            var extendedInfo = dojo.clone(extendedDisplayInfo);
	            if(extendedInfo.itemActions && extendedInfo.itemActions.length){
	            	dojo.forEach(extendedInfo.itemActions, function(itemAction){
	            		itemAction.item = itemData;
	            	});
	            }
	            extendedInfo.actionImage = ImageCache.BAD_LINK;
	            extendedInfo.alwaysShowActionImage = true;
	            
	            var holder = _d.create("div");
	            var nod = imageRenderer.addImageRendering(holder, viewer, cellData, extendedInfo, cellModifier, rowObject, columnIndex, cellWidthHint, true);
	            // Get back link item level action image node
	            var backLinkImgNode = (nod.childNodes && nod.childNodes[1]) ? nod.childNodes[1] : nod;
	            node.appendChild(aLink);
	            _d.place(backLinkImgNode, aLink, useNumeric ? 'before' : 'first');
			} else {
				var decorateImage = this._hasDecorateImage(itemData);			
				var aLink = this._createLinkNode(viewer, cellData, itemData, cellModifier, rowObject, columnIndex, linkWidth, !decorateImage, /*don't add resize marker*/false, useNumeric);
	            if (decorateImage) {
		            var imageRenderer = viewer.getCellRenderer(TableConstants.RENDER_TYPE_IMAGE);
		            var imageNode = imageRenderer.renderCell(viewer, cellData, itemData, cellModifier, rowObject, columnIndex);
					 _d.style(imageNode, {
						'verticalAlign': 'middle',
						'margin': '3px 3px'
					});
		            _d.place(imageNode, aLink, 'first');
	            }
				node.appendChild(aLink);
			}
            
            if (i < atLast) {
                var sep = doc.createTextNode(", ");
                node.appendChild(sep);
            }
        }
        
        if (!nItems) {
            this._renderCellText(viewer, cellData, extendedDisplayInfo, node, cellWidthHint - (hasImage ? IconWidth : 0), /*don't add resize marker*/false);
        }
        else {
            _d.addClass(node, 'rich-hover-list-cell');
        }
        // if we're only showing a single link or no link, make the node inline.
        // Otherwise, the link or the text could wrap if we have a image decorator.
        if(nItems < 2) {
            _d.style(node, 'display', 'inline-block');
        }
        if (hasImage) {
            var imageRenderer = viewer.getCellRenderer(TableConstants.RENDER_TYPE_IMAGE);
            node = imageRenderer.addImageRendering(node, viewer, cellData, extendedDisplayInfo, cellModifier, rowObject, columnIndex, cellWidthHint);
        }
        // allow the wrapper node to resize
        _d.addClass(node, TableConstants.CELL_RESIZE_MARKER_CLASS_NAME);
        return node;
    },
    
    connectEditor: function(tableViewer, cellNode, valueNode, rowObject, columnIndex)
    {
    	_d.addClass(cellNode, "table-cell-editable");
    	return valueNode;
    }
    
});

})();
