/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.rdm.web.project.AddColumnDialog");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.MenuItem");
dojo.require("dijit.MenuSeparator");

dojo.require("com.ibm.rdm.web.artifactgrid.ArtifactConstants");
dojo.require("com.ibm.rdm.web.common.DialogForm");
dojo.require("com.ibm.rdm.web.common.Menu");
dojo.require("com.ibm.rdm.web.model.types.TypesManager");
dojo.require("com.ibm.rdm.web.notifications.ChangeNotifier");
dojo.require("com.ibm.rdm.web.project.ColumnFormatDialog");
dojo.require("com.ibm.rdm.web.suspicion.SuspectLinksUtil");
dojo.require("com.ibm.rdm.web.model.types.RRMTypes");

dojo.requireLocalization("com.ibm.rdm.web.project", "AddColumnDialog");
dojo.requireLocalization("com.ibm.rdm.web.artifactgrid", "ArtifactDisplay");

var TypesManager = com.ibm.rdm.web.model.types.TypesManager,
	RRMTypes = com.ibm.rdm.web.model.types.RRMTypes,
	SuspectLinksUtil = com.ibm.rdm.web.suspicion.SuspectLinksUtil,
	ChangeNotifier = com.ibm.rdm.web.notifications.ChangeNotifier,
	prototype = {

		templatePath: dojo.moduleUrl("com.ibm.rdm.web", "project/templates/AddColumnDialog.html"),
		
		valid: false,
		
		// Attach points
		addButton: null,
		removeButton: null,
		moveUpButton: null,
		moveDownButton: null,
		formatButton: null,
		attributeDiv: null,
		selectedDiv: null,
		
		////  Data Models for the Dialog ////////
		
		
		//We need a data model for all the artifact types and their attributes
		_typesToAttributes: null,
		
		//Map of col-ids to their griddata entry
		_colIdsToGrid: null,
		
		//A list of all the artifact names and URIs
		_types: null,
		
		//An [] for the link types that can be chosen.
		_linkCols: null,
		
		//An [] for the system types that can be chosen.
		_systemCols: null,
		
		//A model for the currently active attributes/link types that can be selected from (which needs to exclude those already chosen)
		_activeAttributes: null,
		
		//We need the selected attributes and ids to return to the caller (and to remove from the .
		_selectedColumns: null,
		
		//Whether we should be showing common attributes only
		_commonOnly: false,
		
		//Map of common attribute columns
		_commonAttributes: null,
		
		//Map of link columns
		_linkColGroup: null,
		
		//Boolean to determine whether we should show the no suspicion profiles selected message when adding suspicion column
		_showNoSuspicionProfilesMessage: null,
		
		_formatDialog: null,
		
		postMixInProperties: function() {
			this.inherited(arguments);
			// This is necessary because of a defect found
			// post-string-freeze in 4.0.1 RC1. This will safely
			// extend the messages dictionary with the
			// ArtifactDisplay strings, and as long as we keep
			// the AddColumnDialog getLocalization call at the
			// end, collisions will be resolved in favor of the 
			// correct strings for this class. --Christian
			this._messages = dojo.mixin(
				dojo.i18n.getLocalization("com.ibm.rdm.web.artifactgrid",
					"ArtifactDisplay"),
				dojo.i18n.getLocalization("com.ibm.rdm.web.project",
					"AddColumnDialog"));
		},
		
		postCreate: function() {

			//Create the Grids
			this.attributesGrid = new com.ibm.rdm.web.grid.Grid({
				fixedHeight: 250
			}, this.attributeDiv);
			this.selectedGrid = new com.ibm.rdm.web.grid.Grid({
					fixedHeight: 250,
					manageFocusNavigation: false,		// let browser do normal focus management
					manageRowSelection: true			// but allow row to take focus and select/deselect.
				}, this.selectedDiv);
			
			// Only allow numeric input
			// GVT WI 62287
			dojo.connect(this.selectedGrid,"onKeyDown",function(evt) {
				var key = evt.which || evt.keyCode;

				if (!evt.shiftKey && !evt.altKey && !evt.ctrlKey &&
				 // numbers   
					 key >= 48 && key <= 57 ||
				 // Numeric keypad
					 key >= 96 && key <= 105 ||
				 // Backspace and Tab and Enter
					key === 8 || key === 9 || key === 13 ||
				 // Home and End
					key === 35 || key === 36 ||
				 // left and right arrows
					key === 37 || key === 39 ||
				 // Del and Ins
					key === 46 || key === 45) {
					 return true;
				 }
				else {
					 if (!evt.ctrlKey){
						 dojo.stopEvent(evt);
					 }
				 }
			});
			
	
			//Setup the grid models
			this._setupGridModelData();
			
			this.selectedArtifacts = {};
			this._selectedCols = [];
			this._selectedColIds = {};
			this._linkCols = [];
			this._allArtifactMenuItemsGroup = {};
			this._showNoSuspicionProfilesMessage = false;
			
			this.filterchars = "";

			this.connect(this.defaultButton, "onclick", function (evt) {
				// This connect must be programmatic in order to stop
				// the event from propagating.
				dojo.stopEvent(evt);
				this.handleResetToDefault();
			});
			
			this.connect(this._sharedField, "onChange", function () {
				this.handleCommonClicked();
			});
			
			//Create the Dijit Menu
			this._artifactMenu = new com.ibm.rdm.web.common.Menu();
			
			// DOJO_ADOPTION
			// Override the 'onClick' method to ensure menu doesn't close.
			// This is a copy of the dojo menu method, with 'this.onExecute' commented out.
			this._artifactMenu.onItemClick = function(/*dijit/_WidgetBase*/ item, /*Event*/ evt){
				// summary:
				//		Handle clicks on an item.
				// tags:
				//		private

				// this can't be done in _onFocus since the _onFocus events occurs asynchronously
				if(typeof this.isShowingNow == 'undefined'){ // non-popup menu
					this._markActive();
				}

				this.focusChild(item);

				if(item.disabled){ return false; }

				if(item.popup){
					this._openPopup(evt.type == "keypress");
				}else{
					// before calling user defined handler, close hierarchy of menus
					// and restore focus to place it was when menu was opened
					//*Amended* We don't want a click to close the menu
					//this.onExecute();

					// user defined handler for click
					item._onClick ? item._onClick(evt) : item.onClick(evt);
				}
			};
			
			this._allArtifactMenuItems = [];
			
			this._systemCols = [];

			this._groupNode = new dijit.form.DropDownButton({
				label: "All",
				dropDown: this._artifactMenu,
				align:"right",
				iconClass:"down-icon",
				title: "All",
				showLabel:"false",
				maxHeight: 340
			}, this._attributeNodeDiv);
			
			var self = this;
			var _myOpenDropDownMethod = this._myOpenDropDown;
			require(
				["dojo/dom-style",
				 "dojo/dom-attr",
				 "dojo/dom-class",
				 "dojo/window",
				 "dojo/dom-geometry",
				 "dijit/popup",
				 "dojo/_base/lang"],
				dojo.hitch(this, function(domStyle, domAttr, domClass, winUtils,
										 domGeometry, popup, lang){
					this._groupNode.openDropDown = dojo.hitch(
						this._groupNode,
						_myOpenDropDownMethod,
						self, domStyle, domAttr, domClass, winUtils, domGeometry, popup, lang
					);
				})
			);
			
			this._createFilterBox();

			this._artifactMenu.maxHeight = 340;
			
			var icon = document.createElement("i");
			icon.className = "common-icon";
			this._sharedFieldImage.appendChild(icon);
		},

		_enableOkButton: function() {
			// summary:
			//		Only enable the Ok button if the user has selected at least 1 attribute
			//		to display.
			if (this.selectedGrid.module.getRowCount() > 0) {
				dojo.attr(this.ok, "disabled", false);
			} else {
				dojo.attr(this.ok, "disabled", true);
			}
		},
		
		// DOJO_ADOPTION
		// This is a copy of the openDropDown function in _HasDropDown, suitably changed to ensure a size is honoured.
		// The proper openDropDown function is replaced with this.
		_myOpenDropDown: function(originalSelf, domStyle, domAttr, domClass, winUtils, domGeometry, popup, lang){
			// summary:
			//		Opens the dropdown for this widget.   To be called only when this.dropDown
			//		has been created and is ready to display (ie, it's data is loaded).
			// returns:
			//		return value of dijit/popup.open()
			// tags:
			//		protected

			var dropDown = this.dropDown,
				ddNode = dropDown.domNode,
				aroundNode = this._aroundNode || this.domNode,
				self = this;

			// Prepare our popup's height and honor maxHeight if it exists.

			// TODO: isn't maxHeight dependent on the return value from dijit/popup.open(),
			// ie, dependent on how much space is available (BK)

			if(!this._preparedNode){
				this._preparedNode = true;
				// Check if we have explicitly set width and height on the dropdown widget dom node
				if(ddNode.style.width){
					this._explicitDDWidth = true;
				}
				if(ddNode.style.height){
					this._explicitDDHeight = true;
				}
			}

			// Code for resizing dropdown (height limitation, or increasing width to match my width)
			if(this.maxHeight || this.forceWidth || this.autoWidth){
				var myStyle = {
					display: "",
					visibility: "hidden"
				};
				if(!this._explicitDDWidth){
					myStyle.width = "";
				}
				if(!this._explicitDDHeight){
					myStyle.height = "";
				}
				domStyle.set(ddNode, myStyle);

				// Figure out maximum height allowed (if there is a height restriction)
				var maxHeight = this.maxHeight;
				if(maxHeight == -1){
					// limit height to space available in viewport either above or below my domNode
					// (whichever side has more room)
					var viewport = winUtils.getBox(this.ownerDocument),
						position = domGeometry.position(aroundNode, false);
					maxHeight = Math.floor(Math.max(position.y, viewport.h - (position.y + position.h)));
				}

				// Attach dropDown to DOM and make make visibility:hidden rather than display:none
				// so we call startup() and also get the size
				popup.moveOffScreen(dropDown);

				if(dropDown.startup && !dropDown._started){
					dropDown.startup(); // this has to be done after being added to the DOM
				}
				// Get size of drop down, and determine if vertical scroll bar needed.  If no scroll bar needed,
				// use overflow:visible rather than overflow:hidden so off-by-one errors don't hide drop down border.
				var mb = domGeometry.getMarginSize(ddNode);
				var overHeight = (maxHeight && mb.h > maxHeight);
				domStyle.set(ddNode, {
					overflowX: "visible",
					overflowY: overHeight ? "auto" : "visible"
				});
				if(overHeight){
					mb.h = maxHeight;
					if("w" in mb){
						mb.w += 16;	// room for vertical scrollbar
					}
				}else{
					delete mb.h;
				}

				// Adjust dropdown width to match or be larger than my width
				if(this.forceWidth){
					mb.w = aroundNode.offsetWidth;
				}else if(this.autoWidth){
					mb.w = Math.max(mb.w, aroundNode.offsetWidth);
				}else{
					delete mb.w;
				}

				// And finally, resize the dropdown to calculated height and width
				if(lang.isFunction(dropDown.resize)){
					dropDown.resize(mb);
				}else{
					domGeometry.setMarginBox(ddNode, mb);
				}
			}

			var retVal = popup.open({
				parent: this,
				popup: dropDown,
				around: aroundNode,
				orient: this.dropDownPosition,
				onExecute: function(){
					self.closeDropDown(true);
				},
				onCancel: function(){
					self.closeDropDown(true);
				},
				onClose: function(){
					self.set("iconClass", "down-icon");
					domAttr.set(self._popupStateNode, "popupActive", false);
					domClass.remove(self._popupStateNode, "dijitHasDropDownOpen");
					self._set("_opened", false);	// use set() because _CssStateMixin is watching
				}
			});
			/* This is the amended code */
			if (overHeight) {
					//Make room for vertical scrollbar.
					if("w" in mb){
						mb.w += 18;
					}
					domGeometry.setMarginBox(dropDown.domNode.parentNode, mb);
					domStyle.set(dropDown.domNode.parentNode, {"overflow": "auto"});
					originalSelf._popupParent = dropDown.domNode.parentNode;
			}
			/* Amended code stop */
			
			domAttr.set(this._popupStateNode, "popupActive", "true");
			domClass.add(this._popupStateNode, "dijitHasDropDownOpen");
			this._set("_opened", true);	// use set() because _CssStateMixin is watching
			
			this._popupStateNode.setAttribute("aria-expanded", "true");
			this._popupStateNode.setAttribute("aria-owns", dropDown.id);
			
			this.set("iconClass", "up-icon");

			// Set aria-labelledby on dropdown if it's not already set to something more meaningful
			if(ddNode.getAttribute("role") !== "presentation" && !ddNode.getAttribute("aria-labelledby")){
				ddNode.setAttribute("aria-labelledby", this.id);
			}
			
			return retVal;
		},
			
		show: function (griddata, returnFunction) {
		
			this.griddata = griddata;
			this.returnFunc = returnFunction;
			
			this.attributesGrid.startup();
			this.attributesGrid.resize({w:200, h:400});
			
			this.selectedGrid.startup();
			this.selectedGrid.resize({w:200, h:400});
		
			this._dialog = new jazz.ui.Dialog({
				contentNode: this.domNode,
				primaryTitle: this._messages.dialogTitle,
				destroyContents: true,
				width: "auto"
			});
			
			this._filterBox.focus();
			
			this.attributesGrid.showArtifacts([]);
			this.selectedGrid.showArtifacts([]);
			
			//Get the TypeSystem information for all All artifacts
			var params = {projectURI: this.projectURI};
			if (this.baselineURI) {
				params.projectSnapshotURI = this.baselineURI;
			}
			
			var objectTypesDeferred =
				TypesManager.getObjectTypesForProject(params);
			
			objectTypesDeferred.addCallback(this, function(objectTypes) {
				this.allDataAvailable(objectTypes);
			});
			
			
		},
		
		allDataAvailable: function(objectTypes) {
			// summary:
			//		Called when all the type system (and other) data is available.
			
			//Add the  info
			var mainMenu = this._artifactMenu;
			var self = this;

			//Add a menu item for All
			var menuItem = new dijit.CheckedMenuItem({
						   label: this._messages.all, 
						   parentMenu: mainMenu,
						   disabled: false,
						   onClick: function () {self.handleClickAll();}
						});
			menuItem.set("menuId", this._messages.all);
			this._allMenuItem = menuItem;

			mainMenu.addChild(menuItem);
			
			//Create the link menu item
			var menuItem = new dijit.CheckedMenuItem({
						   label: this._messages.links, 
						   parentMenu: mainMenu,
						   disabled: false,
						   onClick: function () {self.handleClickLinks();}
						}); 
			menuItem.set("menuId", this._messages.links);
			this._linkMenuItem = menuItem;
			
			
			//Add it to the menu.
			mainMenu.addChild(menuItem);
			mainMenu.addChild(new dijit.MenuSeparator());
			
			var artifactNames = [];
			this._types = [];
			this._typesToAttributes = {};
			
			var globalCommonAttributeURIs = {};
			var first = true;
			var usedByAttributeToArtifactList = {};
			
			//Build a list of available Artifact Types, and the global common attributes (if any)
			for (var i = 0; i < objectTypes.length; i++) {
				var newGlobalCommonAttributeURIs = {};
				var artifact = objectTypes[i];
				var uri = artifact.getResourceURI();
				var name = artifact.getTitle();
				
				artifactNames.push(name);
				this._addArtifact(name, uri);
				
				//Get the attributes for the artifact
				var artifactAttributeURIs = [];
				var attributes = artifact.getAttributeDefinitions();
				for (var j=0; j < attributes.length; j++) {
					artifactAttributeURIs.push(attributes[j]);
					if (first) {
						newGlobalCommonAttributeURIs[attributes[j]] = true;
					} else {
						if (globalCommonAttributeURIs[attributes[j]]) {
							newGlobalCommonAttributeURIs[attributes[j]] = true;
						}
					}
					//Now, add the artifact to the attribute uses list
					var attributeUsedByList = usedByAttributeToArtifactList[attributes[j]];
					if (!attributeUsedByList) {
						//First time use for this attribute
						attributeUsedByList = [];
						usedByAttributeToArtifactList[attributes[j]] = attributeUsedByList;
					} 
					//Add to the existing use of attributes
					attributeUsedByList.push(name);
				}
				

				this._types.push({uri: uri, name: name});
				this._typesToAttributes[uri] = artifactAttributeURIs;
				globalCommonAttributeURIs = newGlobalCommonAttributeURIs;
			}
		

			//Build the attributeURI -> column info, and the list of link cols, and the list of system cols.
			this._systemCols = [];
			this._linkCols = [];
			this._attributeToCol = {};
			this._linkColGroup = {};
			this._colIdsToGrid = {};
			this._selectedCols = [];
			this._selectedColIds = {};
			this._globalCommonAttributes = {};
			
			var descriptions = {};
			for (var i=0; i<this.griddata.length; i++) {
				var col = this.griddata[i];
				col.searchName = col.name.toLowerCase();
				
				var description = "";
				if (col.custom.desc) {
					description = col.custom.desc + " ";
				}
					
				if (col.custom.type === "L" ) {
					this._linkCols.push(col);
					this._linkColGroup[col.columnId] = true;
					descriptions[col.columnId] = description + this._messages.usedByAllArtifactTypes;
				} else if (col.custom.type === "S") {
					this._systemCols.push(col);
					descriptions[col.columnId] = description + this._messages.usedByAllArtifactTypes;
				} else if (col.custom.type === "A") {
					this._attributeToCol[col.custom.uri] = col;
					//Calculate the description
					
					var usedBy = usedByAttributeToArtifactList[col.custom.uri];
					if (usedBy) {
						description += this._messages.usedByArtifactTypes;
						var len = usedBy.length;
						for (var j = 0; j< len; j++) {
							description += " "+usedBy[j];
							if ((j+1)<len) {
								description += ", ";
							}
						}
					} else {
						description += this._messages.notUsedByArtifactTypes;
					}
					descriptions[col.columnId]= description;
					
					if (globalCommonAttributeURIs[col.custom.uri]) {
						this._globalCommonAttributes[col.custom.uri] = true;
					}
				}
				
				if (col.visible.checked) {
					this._selectedCols.push(col);
					this._selectedColIds[col.columnId] = true;
				}
				
				// Trace Attributes format
				var colDisplayKey = this.artifactDisplay
					.getDetailsByColId(col.columnId).key;
				var traceAttrs = this.artifactLinks
					.getTraceabilityAttributesForColumn(colDisplayKey);
				if(traceAttrs) {
					col.format = this.artifactLinks
						.getTraceabilityAttributeColumnFormat(
							colDisplayKey);
					col.format.attributes = dojo.map(traceAttrs,
						function(a) { return a.uri; });
				}
				
				this._colIdsToGrid[col.columnId] = col;
			}
			this._descriptions = descriptions;

			//Now calculate the default items selected
			if (this.artifactTypes) {
				//We have some types - select them by default
				for (var i = 0; i < this.artifactTypes.length; i++) {
					var uri = this.artifactTypes[i];
					var artifactDetails = this._allArtifactMenuItemsGroup[uri];
					if (artifactDetails) {
						artifactDetails.menuItem.set("checked", true);
						this.selectedArtifacts[uri] = artifactDetails.name;
					}
				}
				this._allMenuItem.set("checked", false);
			} else {
				//Default 'All' to on
				this._allMenuItem.set("checked", true);
			}
			
			
			//Init the list
			this._recalcArtifactAttributeDisplay();
			this.recalcSelectedAttributes();
			
			
			//Init the menu
			this._artifactMenu.startup();
		},
		
		_createFilterBox: function() {
			var filterBox = new jazz.ui.FilterBox(
				{
					onChange: dojo.hitch(this, this.handleFilterChange),
					defaultText: this._messages.inBoxFilterMessage
				},
				this._filterBoxDiv
			);
			this._filterBox = filterBox;
		},
		
		
		_addArtifact: function(name, uri) {
			// summary:
			//		Adds the artifact to the drop down menu. 
			// name:
			//		The name/label for the ArtifactType.
			// uri:
			//		The uri of the ArtifactType.

			var mainMenu = this._artifactMenu;
			var self = this;
			
			//Create a menu item
			var menuItem = new dijit.CheckedMenuItem({
						   label: name, 
						   parentMenu: mainMenu,
						   disabled: false,
						   onClick: function () { self.handleClickArtifactType(menuItem, name, uri);}
						}); 
			menuItem.set("menuId", name);
			
			this._allArtifactMenuItems.push(menuItem);
			this._allArtifactMenuItemsGroup[uri] = {menuItem: menuItem, name: name};
			
			//Add it to the menu.
			mainMenu.addChild(menuItem);
		},
		
		_setupGridModelData: function () {

			// Available attribtues column
			var attribute_main_col = {
				name: this._messages.selectMessage,
				width: 160,
				isVisible: true,
				isAlwaysVisible: true,
				recordFunction: dojo.hitch(this, function(key){
					return this.getAttributeRowMainCol(key);
				}),
				isFixedWidth: true
			};
			
			// Selected attributes column. They share 370px
			var selected_name_col = {
				width: 160, 
				name: this._messages.columnsMessage, 
				recordFunction: dojo.hitch(this, function(key){
					return this.getSelectedRowName(key);
				}),
				isVisible: true,
				isAlwaysVisible: true,
				isFixedWidth: true
			};
			
			var selected_width_col = {
				name: this._messages.widthMessage, 
				width: 80, 
				showFullTitleOnHover: true,
				recordFunction: dojo.hitch(this, function(key){
					return this.getSelectedRowWidth(key);
				}),
				varyType: com.ibm.rdm.web.grid.Grid.VARIABLE_NUMBERBOX,
				valueSetFunction: dojo.hitch(this, function(key, value) {
					this._setWidthValue(key, value);
				}),
				isVisible: true,
				isAlwaysVisible: true,
				isFixedWidth: true
			};
			
			var selected_formatting_col = {
				name: this._messages.formattingMessage, 
				width: 130, 
				recordFunction:
					dojo.hitch(this, this._getFormattingForColumn),
				isVisible: true,
				isAlwaysVisible: true,
				isFixedWidth: true	
			};

			//Define the row type
			this.attributesGrid.addRowType("ATTRIBUTE_ROW", {
				isSelectable: true,
				hasSelectionBox: false,
				hasMenu: false,
				isPrimary: true,
				allowsMultiSelect: true,
				rowDataFunction: function(key) {
					return key;
				}
			});

			this.attributesGrid.addColumn("ATTRIBUTE_ROW",
				"attribute_main_col", attribute_main_col);
			
			this.attributesGrid.configureGrid({
				onDoubleClick:
					dojo.hitch(this, this._onHandleDoubleClickAdd)
			});

			this.selectedGrid.addRowType("SELECTED_ROW", {
				isSelectable: true,
				hasSelectionBox: false,
				hasMenu: false,
				isPrimary: true,
				allowsMultiSelect: true,
				rowDataFunction: function(key) {
					return key;
				}
			});
			
			this.selectedGrid.addColumn("SELECTED_ROW",
				"selected_name_col", selected_name_col);
			this.selectedGrid.addColumn("SELECTED_ROW",
				"selected_width_col", selected_width_col);
			this.selectedGrid.addColumn("SELECTED_ROW",
				"selected_formatting_col", selected_formatting_col);
			
			this.selectedGrid.configureGrid({
				onSelectionChange:
					dojo.hitch(this, this._selectedGridSelectionChanged),
				onDoubleClick:
					dojo.hitch(this, this._onHandleDoubleClickRemove)
			});
		},
		
		_selectedGridSelectionChanged: function(selection) {
			if(selection) {
				if(selection.length === 1
					&& selection[0].uri.indexOf("lk") === 0) {
					dojo.attr(this.formatButton, "disabled", false);
				}
				else {
					dojo.attr(this.formatButton, "disabled", true);
				}
			}
		},
		
		_getFormattingForColumn: function(key) {
			var formatDiv = dojo.create("div");
			var renderFormat = function(ofTypeIdx, attributes) {
				attributes[ofTypeIdx] = this._messages.objectType;
				var label = dojo.map(attributes, function(attr) {
					if(typeof attr === "string") {
						return attr;
					}
					return attr.getTitle();
				}).join(", ");
				
				dojo.attr(formatDiv, "innerHTML", label);
			};
			
			if(key && key.indexOf("lk") == 0 && this._colIdsToGrid[key]) {
				if(this._colIdsToGrid[key].format) {
					var attributes = this._colIdsToGrid[key].format.attributes;
					var ofTypeIdx = dojo.indexOf(attributes, RRMTypes.WELL_KNOWN_OBJECT_TYPE);

					renderFormat = dojo.partial(renderFormat, ofTypeIdx);

					com.ibm.rdm.web.model.types.TypesManager
						.getAttributeDefinitions(this.projectURI, attributes)
						.addCallback(this, renderFormat);
				}
			}
			return formatDiv;
		},
		
		_buildReturnGridData: function() {
			// summary:
			//		Builds the griddata to pass back to the control.
			//		Adds in all the visible columns first, then the 'unused' invisible ones.
			var newGridData = [];
			var gd = this.griddata;
			
			//Put the selected ones in the right order at the top
			for(var i =0; i<this._selectedCols.length; i++) {
				var selectedCol = this._selectedCols[i];
				if (selectedCol.columnId === com.ibm.rdm.web.artifactgrid.ArtifactConstants.SUSPICION_COL) {
					var suspicionProfileURIs = SuspectLinksUtil.getSuspicionProfileURIsPreference(this.projectURI);
					if (suspicionProfileURIs.length === 0) {
						this._showNoSuspicionProfilesMessage = true;
					}
				}
				newGridData.push(selectedCol);
			}
			
			//Add the rest underneath
			for (var i=0; i<this.griddata.length; i++) {
				var col = this.griddata[i];
				if (!this._selectedColIds[col.columnId]) {
					newGridData.push(col);
				}
			}
			
			return newGridData;
		},
		
		_recalcArtifactAttributeDisplay: function() {
			// summary:
			//		Called when the artifact list has changed and the list of selected artifacts need to change.
			
			//Recalculate the label for the drop down.
			var label;
			if (this.linksSelected) {
				label = this._messages.links;
			} else {
				var first = true;
				for (var uri in this.selectedArtifacts) {
					if (first) {
						label = this.selectedArtifacts[uri];
						first = false;
					} else {
						label += (", " + this.selectedArtifacts[uri]);
					}
				}
				if (first) {
					//None found
					label = this._messages.all;
				}
			}
			
			var title = label;
			if (label.length > 33) {
				label = label.substr(0,32) + "...";
			}
			
			this._groupNode.set("label", label);
			this._groupNode.set("title", title);
			
			//Now recalculate the list of shown attributes.
			var rowsToShow = {};
			var customCommonAttributes = {};
			
			if (this.linksSelected) {
				for (var i = 0; i< this._linkCols.length; i++) {
					var col = this._linkCols[i];
					rowsToShow[col.columnId] = true;
				}
			} else {
				var found = false;
				var selectedAttributesColl = {};
				for (var uri in this.selectedArtifacts) {
					//Get the attributes for this artifact and add then
					var attributeURIs = this._typesToAttributes[uri];
					
					var newCommon = {};
					
					for (var i = 0; i<attributeURIs.length; i++) {
						var col = this._attributeToCol[attributeURIs[i]];
						selectedAttributesColl[col.columnId] = true;
						
						if (!found) {
							//First artifact - add all to newCommon
							newCommon[col.columnId] = true;
						} else {
							//If second or more - if this col is in common already, move to new common
							if (customCommonAttributes[col.columnId]) {
								newCommon[col.columnId] = true;
							}
						}
					}
					
					//Reset common
					customCommonAttributes = newCommon;
					found = true;
				}
				
				if (found) {
					if (this._commonOnly) {
						//List is 'common' only.				
						for (var colId in customCommonAttributes) {
							rowsToShow[colId]=true;
						}
					} else {
						//For each of the (unified) attributes, add the cols
						for (var colId in selectedAttributesColl) {
							rowsToShow[colId]=true;
						}
					}
				} else {
					//None found - show all
					for (var uri in this._attributeToCol) {
						var col = this._attributeToCol[uri];
						if (!this._commonOnly || this._globalCommonAttributes[col.columnId]) {
							//Add if we are not showing common OR this is a global common attribute
							rowsToShow[col.columnId] = true;
						}
					}
					customCommonAttributes = this._globalCommonAttributes;
					//Add in Links when showing all as well
					for (var i = 0; i< this._linkCols.length; i++) {
						var col = this._linkCols[i];
						rowsToShow[col.columnId] = true;
					}
				}

				this._commonAttributes = customCommonAttributes;

				//Add in System Attribs
				for (var i = 0; i< this._systemCols.length; i++) {
					var col = this._systemCols[i];
					rowsToShow[col.columnId] = true;
					this._commonAttributes[col.columnId] = true;
				}
			}				
	
			//Convert collection to array
			var rowsToShowArr = [];
			var rowsToRefresh = [];
			for (var colId in rowsToShow) {
				var col = this._colIdsToGrid[colId];
				var name = col.searchName || col.name;
				
				//Filter list by the filter characters - continue if we don't match
				//Filter list by those already chosen
				var includesChars = this.filterchars && name.indexOf(this.filterchars)>=0;
				
				if ((!this.filterchars || includesChars) && !this._selectedColIds[colId]) {
					var rowDetails = {rowtype: "ATTRIBUTE_ROW", key: colId};
					rowsToShowArr.push(rowDetails);
					//We need to refresh this row - it may have become or stopped being common
					rowsToRefresh.push(rowDetails);
				}
			}
			if (rowsToRefresh.length > 0) {
				this.attributesGrid.refreshRows(rowsToRefresh, true /*noFocus*/);
			}
			
			//SortList
			var self = this;
			rowsToShowArr.sort(function (a, b){
				var ga = self._colIdsToGrid[a.key];
				var gb = self._colIdsToGrid[b.key];
				return (ga.name < gb.name) ? -1 : 1;
			});
				
			this.attributesGrid.showArtifacts(rowsToShowArr);
		},
		
		recalcSelectedAttributes: function() {
			//Display the selected cols
			var selectedRowsToShowArr = [];
			for (var i=0; i< this._selectedCols.length; i++) {
				selectedRowsToShowArr.push({rowtype: "SELECTED_ROW", key: this._selectedCols[i].columnId});
			}
			
			this.selectedGrid.showArtifacts(selectedRowsToShowArr);
			this._enableOkButton();
		},
		
		
		//// Methods for the possible actions in the dialog.
		
		handleOk: function() {
			var newGridData = this._buildReturnGridData();		
			this.returnFunc(newGridData);	
			
			//Clean the popup that we have hacked to be a certain size...
			if (this._popupParent) {
				dojo.style(this._popupParent, {overflow: "", height: "", width: ""});
			}
			
			if (this._showNoSuspicionProfilesMessage) {
				com.ibm.rdm.web.suspicion.SuspectLinksUtil.showNoProfilesMessage();
			}

			//Ensure the grid is refreshed
			ChangeNotifier.notifyListeners({
				eventTopic: com.ibm.rdm.web.notifications.EventTopics.PAGE_DEFINITION_CHANGE,
				projectURI: this.projectURI,
				resourceUriScope: this.resourceURI,
				columnsChanged: true
			});
			
			this.hide();
		},
		
		handleCancel: function() {
			//Clean the popup that we have hacked to be a certain size...
			if (this._popupParent) {
				dojo.style(this._popupParent, {overflow: "", height: "", width: ""});
			}
			this.hide();
		},
					
		handleResetToDefault: function() {
			// summary:
			//		Used to reset the selected columns to the default set.
			
			//These are the current defaults - do we want these to be parameterised somehow?
			var isModule = (this.viewType === com.ibm.rdm.web.project.DashboardConstants.VIEW_TYPES.MODULE);
			var defaultIds = [];
			var defaultWidths = [];
			if (!isModule) {
				defaultIds = ["id-col", "name", "obj_type_col", "mod-user", "mod-date"];
				defaultWidths = ["25", "260", "157", "207", "252"];
			} else {
				defaultIds = ["id-col", "module_content_col"];
				defaultWidths = ["25", "805"];
			}
			
			//Remove all existing cols
			for (var i=0; i< this._selectedCols.length; i++) {
				var col = this._selectedCols[i];
				col.visible.checked = false;
				delete col.width.set;
				delete this._selectedColIds[col.columnId];
			}
			
			//Add in the default ones
			var newSelectedCols = [];
			for (var i=0; i< defaultIds.length; i++) {
				var col = this._colIdsToGrid[defaultIds[i]];
				
				newSelectedCols.push(col);
				col.visible.checked = true;
				this._selectedColIds[col.columnId]=true;
				this._setWidthValue(defaultIds[i], defaultWidths[i]);
			}
			
			this._selectedCols = newSelectedCols;
			
			this.recalcSelectedAttributes();
			this._recalcArtifactAttributeDisplay();
		},
					
		handleAddClicked: function() {
			// summary:
			//		Adds selected rows on attribute list to selected list
			//		These have to be added to where the currently selected row (the bottom one, if more than one)

			var selected = this.attributesGrid.getSelectedRows();
			
			var currHighlighted = this.selectedGrid.getSelectedRows();
			var insertionPoint = null;
			var beforeInsertion = [];
			var afterInsertion = [];
			var index;
			
			if (currHighlighted && currHighlighted.length > 0) {
				//We have a selection - split the current rows into before and after
				var insertionPointUri = currHighlighted[currHighlighted.length-1].uri;
				var index = -1;
				for(var i = 0; i < this._selectedCols.length; i++) {
					if (insertionPointUri === this._selectedCols[i].columnId) {
						index = i;
					}
				}
				
				for(var i = 0; i < this._selectedCols.length; i++) {
					if (i <= index || index < 0) {
						beforeInsertion.push(this._selectedCols[i]);
					} else {
						afterInsertion.push(this._selectedCols[i]);
					}
				}
			} else {
				beforeInsertion = this._selectedCols;
			}
			
			for (var i=0; i< selected.length; i++) {
				//Add the rows to move onto the end of before
				var row = selected[i];
				var col = this._colIdsToGrid[row.uri];
				
				beforeInsertion.push(col);
				col.visible.checked = true;
				this._selectedColIds[col.columnId]=true;
			}
			
			//And now add the after rows
			for(var i = 0; i < afterInsertion.length; i++) {
				beforeInsertion.push(afterInsertion[i]);
			}
			
			this._selectedCols = beforeInsertion;
			
			//Redisplay everything
			this.recalcSelectedAttributes();
			this._recalcArtifactAttributeDisplay();
		},
					
		handleRemoveClicked: function() {
			// summary:
			//		Removes selected rows on selected list

			var selected = this.selectedGrid.getSelectedRows();
			
			var newSelectedCols = [];
			for (var i=0; i< this._selectedCols.length; i++) {
				var col = this._selectedCols[i];
				var found = false;
				for (var j=0; j< selected.length; j++) {
					if (selected[j].uri === col.columnId) {
						if (!col.visible.disabled) { 
							found = true;
							col.visible.checked = false;
							delete this._selectedColIds[col.columnId];
						}
					}
				}
				//Add the unfound ones back
				if (!found) {
					newSelectedCols.push(col);
				}
			}
			this._selectedCols = newSelectedCols;
			
			this.recalcSelectedAttributes();
			this._recalcArtifactAttributeDisplay();
		},
					
		handleClickArtifactType: function(menuitem, name, uri) {
			//Check the checked state
			var checked = menuitem.checked;
			
			if (checked) {
				this.selectedArtifacts[uri] = name;
			} else {
				delete this.selectedArtifacts[uri];
			}
			
			//Turn off the links value
			this._linkMenuItem.set("checked", false);
			this.linksSelected = false;
			this._allMenuItem.set("checked", false);
			
			this._recalcArtifactAttributeDisplay();
		},
		
		handleClickLinks: function() {
			// summary:
			//		Called when the links entry in the menu has been selected.

			var checked = this._linkMenuItem.get("checked");
			
			//Turns off all attribute selections
			for (var i=0; i < this._allArtifactMenuItems.length; i++) {
				this._allArtifactMenuItems[i].set("checked", false);
			}
			
			this.selectedArtifacts = {};
			this.linksSelected = checked;
			
			if (checked) {
				//Switched links on - ensure All is off
				this._allMenuItem.set("checked", false);
			} else {
				//Selected Links and switched off = make sure All is true
				this._allMenuItem.set("checked", true);
			}
			
			this._recalcArtifactAttributeDisplay();
		},
		
		handleClickAll: function() {
			// summary:
			//		Called when the 'All' entry in the menu has been selected.

			var checked = this._allMenuItem.get("checked");

			this._linkMenuItem.set("checked", false);
			this._allMenuItem.set("checked", true);
			
			//Turns off all attribute selections
			for (var i=0; i < this._allArtifactMenuItems.length; i++) {
				this._allArtifactMenuItems[i].set("checked", false);
			}
			
			this.selectedArtifacts = {};
			this.linksSelected = false;
			
			this._recalcArtifactAttributeDisplay();
		},

		handleFilterChange: function(text) {
			this.filterchars = text;
			this._recalcArtifactAttributeDisplay();
		},
		
		handleCommonClicked: function() {
			this._commonOnly = this._sharedField.get("checked");
			this._recalcArtifactAttributeDisplay();
		},

		handleUpClicked: function() {
			// summary:
			//		Moves the selected rows on the selected list up one place, if possible

			var selected = this.selectedGrid.getSelectedRows();
			
			var newSelectedCols = [];
			var moving = [];
			for (var i=this._selectedCols.length-1; i>=0; i--) {
				var col = this._selectedCols[i];
				var found = false;
				for (var j=0; j< selected.length; j++) {
					if (selected[j].uri === col.columnId) {
						found = true;
					}
				}
				//Add the unfound ones back
				if (!found) {
					newSelectedCols.push(col);
					//And move any moving
					newSelectedCols = newSelectedCols.concat(moving);
					moving = [];
				} else {
					moving.push(col);
				}
			}
			if (moving.length > 0) {
				newSelectedCols = newSelectedCols.concat(moving);
			}
			this._selectedCols = newSelectedCols.reverse();
			
			this.recalcSelectedAttributes();
			
			//Ensure the moved ones are reselected
			var reselected = [];
			for (var j=0; j< selected.length; j++) {
				reselected.push(selected[j].uri);
			}
			
			this.selectedGrid.addToSelection(reselected);
		},
		
		handleDownClicked: function() {
			// summary:
			//		Moves the selected rows on the selected list down one place, if possible

			var selected = this.selectedGrid.getSelectedRows();

			var newSelectedCols = [];
			var moving = [];
			for (var i=0; i<this._selectedCols.length ; i++) {
				var col = this._selectedCols[i];
				var found = false;
				for (var j=0; j< selected.length; j++) {
					if (selected[j].uri === col.columnId) {
						found = true;
					}
				}
				//Add the unfound ones back
				if (!found) {
					newSelectedCols.push(col);
					//And move any moving
					newSelectedCols = newSelectedCols.concat(moving);
					moving = [];
				} else {
					moving.push(col);
				}
			}
			if (moving.length > 0) {
				newSelectedCols = newSelectedCols.concat(moving);
			}
			this._selectedCols = newSelectedCols;
			
			this.recalcSelectedAttributes();
			
			//Ensure the moved ones are reselected
			var reselected = [];
			for (var j=0; j< selected.length; j++) {
				reselected.push(selected[j].uri);
			}
			
			this.selectedGrid.addToSelection(reselected);
			
		},
		
		handleFormatClicked: function() {
			var addFormatDetails = dojo.hitch(this, function(format) {
				// There will be only one selected row for format
				var selected = this.selectedGrid.getSelectedRows()[0];
				this._colIdsToGrid[selected.uri].format = format;
				this.recalcSelectedAttributes();
			});
			var selected = this.selectedGrid.getSelectedRows()[0];
			var format = this._colIdsToGrid[selected.uri].format;
			this._formatDialog =
				new com.ibm.rdm.web.project.ColumnFormatDialog({
				projectURI: this.projectURI,
				baselineURI: this.baselineURI,
				format: format,
				onOk: addFormatDetails,
				onCloseDialog: dojo.hitch(this, function() {
					this._formatDialog.destroyRecursive();
					this._formatDialog = null;
				})
			});
			this._formatDialog.startup();
			this._formatDialog.show();
		},
		
		_onHandleDoubleClickAdd: function() {
			this.handleAddClicked();
		},
		
		_onHandleDoubleClickRemove: function() {
			this.handleRemoveClicked();
		},
		
		//// Methods supporting the attribute value grid.
		
		getAttributeRowMainCol: function(key) {
			// summary:
			//		Builds the div for the entire Attribute value record.
			
			//Intended to be <common/not image> Name with a hover of the description.
			var col = this._colIdsToGrid[key];
			var div = document.createElement("div");
			
			// Add the resource image.
			var iconClass;
			if (this._linkColGroup[key]) {
				iconClass = "link-icon";
			} else if (this._commonAttributes[key]) {
				iconClass = "common-icon";
			} else {
				iconClass = "uncommon-icon";
			}
			var icon = document.createElement("i");
			//img.src = imageURI;
			icon.className = iconClass;
			div.appendChild(icon);

			// Add the name
			var link = document.createElement("span");
			link.innerHTML = col.name;
			if (this._descriptions[key]) {
				dojo.attr(link, "title", this._descriptions[key]);
			} else if (col.custom && col.custom.desc) {
				dojo.attr(link, "title", col.custom.desc);
			}
			div.appendChild(link);
			return div;
		},
		
		//// Methods supporting the selected attributes col.
		getSelectedRowName: function(key) {
			var col = this._colIdsToGrid[key];
			
			var div = document.createElement("div");

			// Add the name
			var link = document.createElement("span");
			link.innerHTML = col.name;
			if (col.custom && col.custom.desc) {
				dojo.attr(div, "title", col.custom.desc);
			}
			div.appendChild(link);
			return div;			
		},
		
		getSelectedRowWidth: function(key) {
			var col = this._colIdsToGrid[key];
			
			return col.width;					
		},
		
		_setWidthValue: function(key, width) {
			var col = this._colIdsToGrid[key];
			
			width = 1*width;
			
			//Ensure width is numeric
			if (isNaN(width) || width < 0 ) {
				width = 0;
			}

			if (col) {
				col.width.value= width;
				col.width.set = true;
			}
		},
		
		uninitialize: function() {
			//Tidies this item
			this.inherited(arguments);
			this.attributesGrid && this.attributesGrid.destroyRecursive();
			this.selectedGrid && this.selectedGrid.destroyRecursive();
			this._artifactMenu && this._artifactMenu.destroyRecursive();
			this._groupNode && this._groupNode.destroyRecursive();
			this._filterBox && this._filterBox.destroyRecursive();
		}
		
	};

dojo.declare("com.ibm.rdm.web.project.AddColumnDialog", com.ibm.rdm.web.common.DialogForm, prototype);
	
})();
